@@ -8,6 +8,7 @@
     @license: GNU GPL, see COPYING for details.
 """
 
+import hashlib
 import re
 import os
 import sys
@@ -27,7 +28,6 @@ from MoinMoin.events import PageRevertedEvent, FileAttachedEvent
 import MoinMoin.web.session
 from MoinMoin.packages import packLine
 from MoinMoin.security import AccessControlList
-from MoinMoin.support.python_compatibility import set
 
 _url_re_cache = None
 _farmconfig_mtime = None
@@ -282,6 +282,12 @@ class ConfigFunctionality(object):
                      Please change it in your wiki configuration and try again."""
             raise error.ConfigurationError(msg)
 
+        if not isinstance(self.actions_superuser, list):
+            msg = """The actions_superuser setting in your wiki configuration is not a list
+                     (e.g. ['newaccount', 'some_other_action']).
+                     Please change it in your wiki configuration and try again."""
+            raise error.ConfigurationError(msg)
+
         # moin < 1.9 used cookie_lifetime = <float> (but converted it to int) for logged-in users and
         # anonymous_session_lifetime = <float> or None for anon users
         # moin >= 1.9 uses cookie_lifetime = (<float>, <float>) - first is anon, second is logged-in
@@ -427,6 +433,24 @@ class ConfigFunctionality(object):
                 raise error.ConfigurationError("You must set a (at least %d chars long) secret string for secrets['%s']!" % (
                     secret_min_length, secret_key_name))
 
+        if self.password_scheme not in config.password_schemes_configurable:
+            raise error.ConfigurationError("not supported: password_scheme = %r" % self.password_scheme)
+
+        if self.passlib_support:
+            try:
+                from passlib.context import CryptContext
+            except ImportError, err:
+                raise error.ConfigurationError("Wiki is configured to use passlib, but importing passlib failed [%s]!" % str(err))
+            try:
+                self.cache.pwd_context = CryptContext(**self.passlib_crypt_context)
+            except (ValueError, KeyError, TypeError, UserWarning), err:
+                # ValueError: wrong configuration values
+                # KeyError: unsupported hash (seen with passlib 1.3)
+                # TypeError: configuration value has wrong type
+                raise error.ConfigurationError("passlib_crypt_context configuration is invalid [%s]." % str(err))
+        elif self.password_scheme == '{PASSLIB}':
+            raise error.ConfigurationError("passlib_support is switched off, thus you can't use password_scheme = '{PASSLIB}'.")
+
     def calc_secrets(self):
         """ make up some 'secret' using some config values """
         varnames = ['data_dir', 'data_underlay_dir', 'language_default',
@@ -542,7 +566,8 @@ file. It should match the actual charset of the configuration file.
             'page_front_page', 'page_category_regex', 'page_dict_regex',
             'page_group_regex', 'page_template_regex', 'page_license_page',
             'page_local_spelling_words', 'acl_rights_default',
-            'acl_rights_before', 'acl_rights_after', 'mail_from'
+            'acl_rights_before', 'acl_rights_after', 'mail_from',
+            'quicklinks_default', 'subscribed_pages_default',
             )
 
         for name in decode_names:
@@ -607,7 +632,6 @@ also the spelling of the directory name.
         plugin packages as "moin_plugin_<sha1(path)>.plugin".
         """
         import imp
-        from MoinMoin.support.python_compatibility import hash_new
 
         plugin_dirs = [self.plugin_dir] + self.plugin_dirs
         self._plugin_modules = []
@@ -617,7 +641,7 @@ also the spelling of the directory name.
             imp.acquire_lock()
             try:
                 for pdir in plugin_dirs:
-                    csum = 'p_%s' % hash_new('sha1', pdir).hexdigest()
+                    csum = 'p_%s' % hashlib.new('sha1', pdir).hexdigest()
                     modname = '%s.%s' % (self.siteid, csum)
                     # If the module is not loaded, try to load it
                     if not modname in sys.modules:
@@ -675,8 +699,11 @@ class DefaultConfig(ConfigFunctionality):
     # the WikiConfig macro. Settings must be added below to
     # the options dictionary.
 
+_default_backlink_method = lambda cfg, req: 'backlink' if req.user.valid else 'pagelink'
 
-def _default_password_checker(cfg, request, username, password):
+
+def _default_password_checker(cfg, request, username, password,
+                              min_length=6, min_different=4):
     """ Check if a password is secure enough.
         We use a built-in check to get rid of the worst passwords.
 
@@ -690,9 +717,9 @@ def _default_password_checker(cfg, request, username, password):
     """
     _ = request.getText
     # in any case, do a very simple built-in check to avoid the worst passwords
-    if len(password) < 6:
+    if len(password) < min_length:
         return _("Password is too short.")
-    if len(set(password)) < 4:
+    if len(set(password)) < min_different:
         return _("Password has not enough different characters.")
 
     username_lower = username.lower()
@@ -771,11 +798,15 @@ options_no_group_name = {
     ('SecurityPolicy',
      None,
      "Class object hook for implementing security restrictions or relaxations"),
+    ('actions_superuser',
+     ['newaccount',  # spam bots create tons of user accounts, so better allow it only for superuser
+     ],
+     "Restrict actions to superuser only (list of strings)"),
     ('actions_excluded',
      ['xmlrpc',  # we do not want wiki admins unknowingly offering xmlrpc service
       'MyPages',  # only works when used with a non-default SecurityPolicy (e.g. autoadmin)
       'CopyPage',  # has questionable behaviour regarding subpages a user can't read, but can copy
-     ],
+      ],
      "Exclude unwanted actions (list of strings)"),
 
     ('allow_xslt', False,
@@ -784,6 +815,37 @@ options_no_group_name = {
     ('password_checker', DefaultExpression('_default_password_checker'),
      'checks whether a password is acceptable (default check is length >= 6, at least 4 different chars, no keyboard sequence, not username used somehow (you can switch this off by using `None`)'),
 
+    ('password_scheme', '{PASSLIB}',
+     'Either "{PASSLIB}" (default) to use passlib for creating and upgrading password hashes (see also passlib_crypt_context for passlib configuration), '
+     'or "{SSHA}" (or any other of the builtin password schemes) to not use passlib (not recommended).'),
+
+    ('passlib_support', True,
+     'If True (default), import passlib and support password hashes offered by it.'),
+
+    ('passlib_crypt_context', dict(
+        # schemes we want to support (or deprecated schemes for which we still have
+        # hashes in our storage).
+        # note: bcrypt: we did not include it as it needs additional code (that is not pure python
+        #       and thus either needs compiling or installing platform-specific binaries) and
+        #       also there was some bcrypt issue in passlib < 1.5.3.
+        #       pbkdf2_sha512: not included as it needs at least passlib 1.4.0
+        #       sha512_crypt: supported since passlib 1.3.0 (first public release)
+        schemes=["sha512_crypt", ],
+        # default scheme for creating new pw hashes (if not given, passlib uses first from schemes)
+        #default="sha512_crypt",
+        # deprecated schemes get auto-upgraded to the default scheme at login
+        # time or when setting a password (including doing a moin account pwreset).
+        # for passlib >= 1.6, giving ["auto"] means that all schemes except the default are deprecated:
+        #deprecated=["auto"],
+        # to support also older passlib versions, rather give a explicit list:
+        #deprecated=[],
+        # vary rounds parameter randomly when creating new hashes...
+        #all__vary_rounds=0.1,
+    ),
+    "passlib CryptContext arguments, see passlib docs"),
+
+    ('recovery_token_lifetime', 12,
+     'how long the password recovery token is valid [h]'),
   )),
   # ==========================================================================
   'spam_leech_dos': ('Anti-Spam/Leech/DOS',
@@ -806,6 +868,11 @@ options_no_group_name = {
         # (like photo galleries) triggering surge protection, we assign rather high limits:
         'AttachFile': (300, 30),
         'cache': (600, 30), # cache action is very cheap/efficient
+        # special stuff to prevent someone trying lots of usernames / passwords to log in.
+        # we keep this commented / disabled so that this feature does not get activated by default
+        # (if somebody does not override surge_action_limits with own values):
+        #'auth-ip': (10, 3600),  # same remote ip (any name)
+        #'auth-name': (10, 3600),  # same name (any remote ip)
      },
      "Surge protection tries to deny clients causing too much load/traffic, see HelpOnConfiguration/SurgeProtection."),
     ('surge_lockout_time', 3600, "time [s] someone gets locked out when ignoring the warnings"),
@@ -824,7 +891,7 @@ options_no_group_name = {
     # and receive a FORBIDDEN for anything except viewing a page
     # list must not contain 'java' because of twikidraw wanting to save drawing uses this useragent
     ('ua_spiders',
-     ('archiver|cfetch|charlotte|crawler|gigabot|googlebot|heritrix|holmes|htdig|httrack|httpunit|'
+     ('archiver|bingbot|cfetch|charlotte|crawler|gigabot|googlebot|heritrix|holmes|htdig|httrack|httpunit|'
       'intelix|jeeves|larbin|leech|libwww-perl|linkbot|linkmap|linkwalk|litefinder|mercator|'
       'microsoft.url.control|mirror| mj12bot|msnbot|msrbot|neomo|nutbot|omniexplorer|puf|robot|scooter|seekbot|'
       'sherlock|slurp|sitecheck|snoopy|spider|teleport|twiceler|voilabot|voyager|webreaper|wget|yeti'),
@@ -897,6 +964,9 @@ options_no_group_name = {
     ('show_version', False, "show moin's version at the bottom of a page"),
     ('show_rename_redirect', False, "if True, offer creation of redirect pages when renaming wiki pages"),
 
+    ('backlink_method', DefaultExpression('_default_backlink_method'),
+     "function determining how the (last part of the) pagename should be rendered in the title area"),
+
     ('packagepages_actions_excluded',
      ['setthemename',  # related to questionable theme stuff, see below
       'copythemefile', # maybe does not work, e.g. if no fs write permissions or real theme file path is unknown to moin
@@ -945,8 +1015,8 @@ options_no_group_name = {
   # ==========================================================================
   'editor': ('Editor related', None, (
     ('editor_default', 'text', "Editor to use by default, 'text' or 'gui'"),
-    ('editor_force', False, "if True, force using the default editor"),
-    ('editor_ui', 'freechoice', "Editor choice shown on the user interface, 'freechoice' or 'theonepreferred'"),
+    ('editor_force', True, "if True, force using the default editor"),
+    ('editor_ui', 'theonepreferred', "Editor choice shown on the user interface, 'freechoice' or 'theonepreferred'"),
     ('page_license_enabled', False, 'if True, show a license hint in page editor.'),
     ('page_license_page', u'WikiLicense', 'Page linked from the license hint. [Unicode]'),
     ('edit_locking', 'warn 10', "Editor locking policy: `None`, `'warn <timeout in minutes>'`, or `'lock <timeout in minutes>'`"),
@@ -1060,10 +1130,12 @@ options_no_group_name = {
 
     ('log_remote_addr', True,
      "if True, log the remote IP address (and maybe hostname)."),
-    ('log_reverse_dns_lookups', True,
-     "if True, do a reverse DNS lookup on page SAVE. If your DNS is broken, set this to False to speed up SAVE."),
+    ('log_reverse_dns_lookups', False,
+     "if True, do a reverse DNS lookup on page SAVE."),
     ('log_timing', False,
      "if True, add timing infos to the log output to analyse load conditions"),
+    ('log_events_format', 1,
+     "0 = no events logging, 1 = standard format (like <= 1.9.7) [default], 2 = extended format"),
 
     # some dangerous mimetypes (we don't use "content-disposition: inline" for them when a user
     # downloads such attachments, because the browser might execute e.g. Javascript contained
@@ -1106,6 +1178,7 @@ options_no_group_name = {
     ('search_results_per_page', 25, "Number of hits shown per page in the search results"),
 
     ('siteid', 'default', None),
+    ('xmlrpc_overwrite_user', True, "Overwrite authenticated user at start of xmlrpc code"),
   )),
 }
 
@@ -1137,7 +1210,7 @@ options = {
     'ACLs control who may do what, see HelpOnAccessControlLists.',
     (
       ('hierarchic', False, 'True to use hierarchical ACLs'),
-      ('rights_default', u"Trusted:read,write,delete,revert Known:read,write,delete,revert All:read,write",
+      ('rights_default', u"Trusted:read,write,delete,revert Known:read All:read",
        "ACL used if no ACL is specified on the page"),
       ('rights_before', u"",
        "ACL that is processed before the on-page/default ACL"),
@@ -1270,6 +1343,58 @@ options = {
       ('import_pagename_envelope', u"%s", "Use this to add some fixed prefix/postfix to the generated target pagename."),
       ('import_pagename_regex', r'\[\[([^\]]*)\]\]', "Regular expression used to search for target pagename specification."),
       ('import_wiki_addrs', [], "Target mail addresses to consider when importing mail"),
+
+      ('notify_page_text', '%(intro)s%(difflink)s\n\n%(comment)s%(diff)s',
+       "Template for putting together the pieces for the page changed/deleted/renamed notification mail text body"),
+      ('notify_page_changed_subject', _('[%(sitename)s] %(trivial)sUpdate of "%(pagename)s" by %(username)s'),
+       "Template for the page changed notification mail subject header"),
+      ('notify_page_changed_intro',
+       _("Dear Wiki user,\n\n"
+         'You have subscribed to a wiki page or wiki category on "%(sitename)s" for change notification.\n\n'
+         'The "%(pagename)s" page has been changed by %(editor)s:\n'),
+       "Template for the page changed notification mail intro text"),
+      ('notify_page_deleted_subject', _('[%(sitename)s] %(trivial)sUpdate of "%(pagename)s" by %(username)s'),
+       "Template for the page deleted notification mail subject header"),
+      ('notify_page_deleted_intro',
+       _("Dear wiki user,\n\n"
+         'You have subscribed to a wiki page "%(sitename)s" for change notification.\n\n'
+         'The page "%(pagename)s" has been deleted by %(editor)s:\n\n'),
+       "Template for the page deleted notification mail intro text"),
+      ('notify_page_renamed_subject', _('[%(sitename)s] %(trivial)sUpdate of "%(pagename)s" by %(username)s'),
+       "Template for the page renamed notification mail subject header"),
+      ('notify_page_renamed_intro',
+       _("Dear wiki user,\n\n"
+         'You have subscribed to a wiki page "%(sitename)s" for change notification.\n\n'
+         'The page "%(pagename)s" has been renamed from "%(oldname)s" by %(editor)s:\n'),
+       "Template for the page renamed notification mail intro text"),
+      ('notify_att_added_subject', _('[%(sitename)s] New attachment added to page %(pagename)s'),
+       "Template for the attachment added notification mail subject header"),
+      ('notify_att_added_intro',
+       _("Dear Wiki user,\n\n"
+         'You have subscribed to a wiki page "%(page_name)s" for change notification. '
+         "An attachment has been added to that page by %(editor)s. "
+         "Following detailed information is available:\n\n"
+         "Attachment name: %(attach_name)s\n"
+         "Attachment size: %(attach_size)s\n"),
+       "Template for the attachment added notification mail intro text"),
+      ('notify_att_removed_subject', _('[%(sitename)s] Removed attachment from page %(pagename)s'),
+       "Template for the attachment removed notification mail subject header"),
+      ('notify_att_removed_intro',
+       _("Dear Wiki user,\n\n"
+         'You have subscribed to a wiki page "%(page_name)s" for change notification. '
+         "An attachment has been removed from that page by %(editor)s. "
+         "Following detailed information is available:\n\n"
+         "Attachment name: %(attach_name)s\n"
+         "Attachment size: %(attach_size)s\n"),
+       "Template for the attachment removed notification mail intro text"),
+      ('notify_user_created_subject',
+       _("[%(sitename)s] New user account created"),
+       "Template for the user created notification mail subject header"),
+      ('notify_user_created_intro',
+       _('Dear Superuser, a new user has just been created on "%(sitename)s". Details follow:\n\n'
+         '    User name: %(username)s\n'
+         '    Email address: %(useremail)s'),
+       "Template for the user created notification mail intro text"),
     )),
 
     'backup': ('Backup settings',
