@@ -171,6 +171,13 @@ class MoinScript(Script):
             help="wiki page name [default: all pages]"
         )
 
+    def _update_option_help(self, opt_string, help_msg):
+        """ Update the help string of an option. """
+        for option in self.parser.option_list:
+            if option.get_opt_string() == opt_string:
+                option.help = help_msg
+                break
+
     def init_request(self):
         """ create request """
         from MoinMoin.web.contexts import ScriptContext
@@ -201,6 +208,7 @@ moin ... cli show ...
 moin ... export dump ...
 
 moin ... import irclog ...
+moin ... import wikipage ...
 
 moin ... index build ...
 
@@ -235,6 +243,11 @@ Specific options:
 """)
 
         cmd_module, cmd_name = args[:2]
+        if cmd_module == '...':
+            # our docs usually tell to use moin ... cmd_module cmd_name
+            # if somebody enters the ... verbatim, tell him how to do it right:
+            fatal("Wrong invokation. Please do not enter ... verbatim, but give --config-dir and --wiki-url options (see help for more details).")
+
         from MoinMoin import wikiutil
         try:
             plugin_class = wikiutil.importBuiltinPlugin('script.%s' % cmd_module, cmd_name, 'PluginScript')
