@@ -12,6 +12,7 @@
 
 import cgi
 import codecs
+import hmac, hashlib
 import os
 import re
 import time
@@ -21,11 +22,11 @@ from MoinMoin import log
 logging = log.getLogger(__name__)
 
 from MoinMoin import config
-from MoinMoin.support.python_compatibility import rsplit
 from inspect import getargspec, isfunction, isclass, ismethod
 
 from MoinMoin import web # needed so that next lines work:
 import werkzeug
+from werkzeug.security import safe_str_cmp as safe_str_equal
 from MoinMoin.util import pysupport, lock
 
 # Exceptions
@@ -120,7 +121,7 @@ def url_unquote(s, want_unicode=None):
 def parseQueryString(qstr, want_unicode=None):
     """ see werkzeug.url_decode
 
-        Please note: this returns a MultiDict, you might need to use dict() on
+        Please note: this returns a MultiDict, you might need to use .to_dict() on
                      the result if your code expects a "normal" dict.
     """
     try:
@@ -176,7 +177,10 @@ def quoteWikinameURL(pagename, charset=config.charset):
     return werkzeug.url_quote(pagename, charset=charset, safe='/')
 
 
-escape = werkzeug.escape
+def escape(s, quote=None):
+    if not isinstance(s, (str, unicode)):
+        s = str(s)
+    return cgi.escape(s, quote)
 
 
 def clean_input(text, max_len=201):
@@ -2266,9 +2270,9 @@ def taintfilename(basename):
     @rtype: string
     @return: (safer) filename
     """
-    for x in (os.pardir, ':', '/', '\\', '<', '>'):
-        basename = basename.replace(x, '_')
-
+    # note: filenames containing ../ (or ..\) are made safe by replacing
+    # the / (or the \). the .. will be kept, but is harmless then.
+    basename = re.sub('[\x00-\x1f:/\\\\<>"*?%|]', '_', basename)
     return basename
 
 
@@ -2471,7 +2475,7 @@ def split_anchor(pagename):
           pagename part (and no anchor). Also, we need to append #anchor
           at the END of the generated URL (AFTER the query string).
     """
-    parts = rsplit(pagename, '#', 1)
+    parts = pagename.rsplit('#', 1)
     if len(parts) == 2:
         return parts
     else:
@@ -2496,8 +2500,6 @@ def createTicket(request, tm=None, action=None, pagename=None):
                              page than the current one, you MUST specify the
                              page name you use when posting the form.
     """
-
-    from MoinMoin.support.python_compatibility import hmac_new
     if tm is None:
         # for age-check of ticket
         tm = "%010x" % time.time()
@@ -2530,9 +2532,9 @@ def createTicket(request, tm=None, action=None, pagename=None):
             value = value.encode('utf-8')
         hmac_data.append(value)
 
-    hmac = hmac_new(request.cfg.secrets['wikiutil/tickets'],
-                    ''.join(hmac_data))
-    return "%s.%s" % (tm, hmac.hexdigest())
+    h = hmac.new(request.cfg.secrets['wikiutil/tickets'],
+                 ''.join(hmac_data), digestmod=hashlib.sha1)
+    return "%s.%s" % (tm, h.hexdigest())
 
 
 def checkTicket(request, ticket):
@@ -2553,7 +2555,7 @@ def checkTicket(request, ticket):
     #       if the ticket was created within a session.
     ourticket = createTicket(request, timestamp_str)
     logging.debug("checkTicket: returning %r, got %r, expected %r" % (ticket == ourticket, ticket, ourticket))
-    return ticket == ourticket
+    return safe_str_equal(ticket, ourticket)
 
 
 def renderText(request, Parser, text):
